MEMORY ; #######################################################
 ; INTERRUPT SERVICE ROUTINE TO DISPLAY MEMORY AVAILABLE
 ; CONSTANTLY ON TOP RIGHT HAND CORNER OF SCREEN IN BASIC
 ; #######################################################
 ;
 	ORG	7000H
 ;
 ; #######################################################
 ; SETUP ROUTINE PATCHES JUMP TO SERVICE ROUTINE & RETURNS
 ; #######################################################
 ;
 SETUP	DI			; INTERRUPTS OFF NOW
 	LD	A,0C3H		; PUT JUMP IN PLACE
 	LD	(4012H),A	; OF PRESENT RETURN (C9)
 	LD	HL,SERVE	; GET SERVICE ROUTINE
 	LD	(4013H),HL	; PUT IT IN PLACE AT JUMP
 	LD	HL,06CCH	; GET BASIC READY ADDR.
 	PUSH	HL		; MAKE IT A RETURN ADDR.
 	IM	1		; SET INTERRUPT MODE
 	EI			; ENABLE INTERRUPTS
 	RET			; RETURN TO BASIC READY
 ;
 ; #######################################################
 ; PARAMETER DEFINITIONS AND SERVICE ROUTINE FOLLOWS
 ; #######################################################
 ;
 VIDEO	EQU	3C00H		; VIDEO SCREEN LOCATION
 STKBOT	EQU	40E8H		; CURRENT BASIC STACK
 ARRTOP	EQU	40FBH		; CURRENT ARRAY TOP
 SERVE	DI			; SERVICE - INTER. OFF
 	PUSH	AF		; SAVE CURRENT A VALUE
 	PUSH	HL		; SAVE CURRENT HL VALUE
 	PUSH	DE		; SAVE CURRENT DE VALUE
 	PUSH	BC		; SAVE CURRENT BC VALUE
 	PUSH	IX		; SAVE CURRENT IX VALUE
 	PUSH	IY		; SAVE CURRENT IY VALUE
 	LD	A,(37ECH)	; READ & RESET INT. LATCH
 	LD	A,(37E0H)	; READ & RESET INT. LATCH
 	LD	HL,MESSG	; GET MESSAGE "MEM="
 	LD	DE,VIDEO+36H	; GET PLACE TO DISPLAY IT
 	LD	BC,5		; CHARACTERS TO DISPLAY
 	LDIR			; DISPLAY MEM MESSAGE
 	XOR	A		; CLEAR VALUE FROM ACC.
 ;
 ; #######################################################
 ; ROUTINE BELOW DOES DIVISION BY SUBTRACTION - KEEP SUB-
 ; TRACTING UNTIL THE VALUE GOES BELOW ZERO, AND COUNT
 ; EACH SUBTRACTION.  WHEN THE VALUE GOES BELOW ZERO, BACK
 ; OFF BY ONE, AND RESTORE THE VALUE SO IT IS POSITIVE.
 ; THIS IS A REMAINDER, WHICH IS THEN DIVIDED BY THE NEXT
 ; SMALLEST ELEMENT IN THE TENS-TABLE.  EACH TIME THE
 ; VALUE DROPS BELOW ZERO, THE COUNTER HAS COUNTED UP TO
 ; THE NUMBER OF TIMES ONE ELEMENT OF THE TENS-TABLE GOES
 ; INTO THE TARGET NUMBER.  GOOD OLD FOURTH GRADE STUFF.
 ; #######################################################
 ;
 	LD	HL,(STKBOT)	; GET TOP MEM VALUE
 	LD	DE,(ARRTOP)	; GET BOTTOM MEM VALUE
 	SBC	HL,DE		; SUBTRACT FOR FREE MEM
 	LD	IX,VIDEO+3BH	; GET DISPLAY POSITIONS
 	LD	IY,TENTAB	; GET TENS-TABLE CONVER.
 	LD	B,5		; NUMBER OF CHARACTERS
 LOOP0	XOR	A		; CLEAR ACCUMULATOR
 	LD	E,(IY+0)	; GET FIRST IN 10-TABLE
 	LD	D,(IY+1)	; GET SECOND IN 10-TABLE
 LOOP1	OR	A		; CHECK FOR REMAINDER
 	SBC	HL,DE		; GET DIFFERENCE LEFT
 	JR	C,JUMP0		; IF <0 THEN GO TO NEXT
 	INC	A		; ELSE A = A + 1
 	JR	LOOP1		; KEEP GOING SUBTRACTIVE
 JUMP0	ADD	HL,DE		; RESTORE FOR POS. VALUE
 	ADD	A,30H		; CONVERT VALUE TO ASCII
 	LD	(IX+0),A	; DISPLAY IT ON SCREEN
 	INC	IX		; GET NEXT SCREEN POS'N
 	INC	IY		; GET NEXT TENS-TABLE
 	INC	IY		;   POSITION (IY+2)
 	DJNZ	LOOP0		; AND DO IT FOR 5 CHARS
 	POP	IY		; RESTORE IY REGISTER
 	POP	IX		; RESTORE IX REGISTER
 	POP	BC		; RESTORE BC REGISTER
 	POP	DE		; RESTORE DE REGISTER
 	POP	HL		; RESTORE HL REGISTER
 	POP	AF		; RESTORE AF REGISTER
 	EI			; INTERRUPTS BACK ON
 	RET			; RETURN FROM INTERRUPT
 ;
 ; #######################################################
 ; MESSAGE TO DISPLAY AND TENS-TABLE FOR HEX-DEC CONVERT.
 ; #######################################################
 ;
 MESSG	DEFB	8FH		; DEFINE BLOCK CHARACTER
 	DEFM	'MEM='		; AND REST OF MESSAGE
 TENTAB	DEFW	10000D		; TENS TABLE ....10[4
 	DEFW	01000D		; 	....10[3
 	DEFW	00100D		;	....10[2
 	DEFW	00010D		; 	....10[1
 	DEFW	00001D		;	....10[0
 ;
 ; #######################################################
 	END	SETUP
